/***************************************************************************
                          missionselector  -  description
                             -------------------
    begin                : Fri Jan 30 2004
    copyright            : (C) 2004, 2005, 2007 by Thomas Friedrichsmeier
    email                : tfry@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef MISSIONSELECTOR_H
#define MISSIONSELECTOR_H

#include <qmap.h>

#include <kdialogbase.h>

#include "tpbase.h"
#include "savegames.h"

class QPushButton;
class QListView;
class QListViewItem;
class QTextEdit;
class QCheckBox;
class QString;
class QTabWidget;
class MissionWidget;
class SavesWidget;
class QLabel;
class QCloseEvent;

/**
Allow selection of (and scanning for) mission(s)/saved game to play.

@author Thomas Friedrichsmeier,,,
*/
class MissionSelector : public KDialogBase, public TPBase {
	Q_OBJECT
public:
	enum Face { Mission=0, Saves=1 };
/** constructor. If scan, scan for new missions */
	MissionSelector (bool scan_missions=false, Face show_face=Mission, bool force_selection=false);
/** destructor */
	~MissionSelector();

public slots:
/** the selection changed. (De-)activate buttons accordingly */
	void selectionChanged (Face in_face, bool valid);
/** the widget (Mission/Saves) changed. (De-)activate buttons accordingly */
	void pageChanged (QWidget *);
protected slots:
/** the ok Button was pressed. Apply settings */
	void slotOk ();
protected:
	void closeEvent (QCloseEvent *e);
private:
	MissionWidget *mwidget;
	SavesWidget *swidget;

	bool prevent_close;

	QTabWidget *tabbook;
	QCheckBox *select_on_startup_box;
};



/** This is the widget that lets you select a mission */
class MissionWidget : public QWidget, public TPBase {
	Q_OBJECT
public:
/** constructor */
	MissionWidget (QWidget *parent, MissionSelector *m_sel);
/** destructor */
	~MissionWidget ();

	void apply ();
public slots:
/** the add Button was pressed. Prompt for single mission(s) to add */
	void addButtonPressed ();
/** the scan Button was pressed. Prompt for directory to scan */
	void scanButtonPressed ();
/** the del Button was pressed. Remove selected missoin(s) from list */
	void delButtonPressed ();
/** the selection changed. (De-)activate buttons accordingly */
	void selectionChanged ();
private:
friend class MissionSelector;
/** read in basic descriptions for all known missions */
	void initialize ();
/** add a single mission */
	void addMission (const QString & filename);

	QLabel *mission_image;
	QTextEdit *description;
	QListView *missionlist;

	QPushButton *delbutton;

	MissionSelector *parent;
};



/** This is the widget that lets you select a saved game */
class SavesWidget : public QWidget, public TPBase {
	Q_OBJECT
public:
/** constructor */
	SavesWidget (QWidget *parent, MissionSelector *m_sel);
/** destructor */
	~SavesWidget ();

	void apply ();
public slots:
/** the del Button was pressed. Remove selected saved games from list */
	void delButtonPressed ();
/** the selection changed. (De-)activate buttons accordingly */
	void selectionChanged ();
private:
	void initialize ();
friend class MissionSelector;
	QTextEdit *description;
	QListView *saveslist;

	QPushButton *delbutton;
	
	MissionSelector *parent;
	
	QMap<QListViewItem*, SaveGames::GameDescription*> smap;
	QPtrList<SaveGames::GameDescription> *saves_pointers;
};

#endif
