/***************************************************************************
                          taxipilotapp.cpp  -  description
                             -------------------
    begin                : Sun Aug 12 2001
    copyright            : (C) 2001, 2002, 2004, 2005, 2007 by Thomas Friedrichsmeier
    email                : Thomas.Friedrichsmeier@ruhr-uni-bochum.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <kstdaction.h>
#include <kkeydialog.h>
#include <kmenubar.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <ktip.h>

#include "taxipilotapp.h"
#include "preferences.h"
#include "message.h"
#include "kaction.h"

#include <qpixmapcache.h>

TaxipilotApp::TaxipilotApp(QWidget * parent, const char *name):KMainWindow(parent, name)
{
	tp = new Taxipilot(this);
	setCentralWidget(tp);

// create the menubar
	gameNew = KStdAction::openNew (tp, SLOT (newGame ()), actionCollection (), "game_new");
	gameAbort = KStdAction::close (tp, SLOT (ask_abortGame ()), actionCollection (), "game_close");
	gameAbort->setEnabled (false);
	gameAbort->setText (i18n ("Stop"));
	gamePause = new KAction (i18n ("&Pause"), 0, tp, SLOT (pauseGame ()), actionCollection (), "game_pausex");
	gamePause->setEnabled (false);
	gameSave = KStdAction::save (tp, SLOT (saveGame ()), actionCollection (), "game_save");
	gameSave->setEnabled (false);
	gameLoad = KStdAction::open (tp, SLOT (loadGame ()), actionCollection (), "game_load");
	gameLoad->setText (i18n ("Load"));
	KStdAction::tipOfDay (this, SLOT (tipOfDay ()), actionCollection ());

	new KAction (i18n ("Show &Highscores..."), 0, tp, SLOT (showHighscores ()), actionCollection (), "game_hiscores");
	KStdAction::quit (tp, SLOT (quitGame ()), actionCollection (), "game_quit");

	KStdAction::preferences (this, SLOT(changePrefs ()), actionCollection ());
	(void) new KAction (i18n ("Select &Mission..."), 0, tp, SLOT (selectMission ()), actionCollection (), "settings_mission");

	KStdAction::keyBindings (this, SLOT (configureKeys ()), actionCollection ());

#if !KDE_IS_VERSION(3,1,0)
	accel = new KAccel(this);

	accel->insertItem(i18n("Accelerate Upwards"), "Upx", Qt::Key_Up);
	accel->insertItem(i18n("Accelerate Leftwards"), "Left", Qt::Key_Left);
	accel->insertItem(i18n("Accelerate Downwards"), "Down", Qt::Key_Down);
	accel->insertItem(i18n("Accelerate Rightwards"), "Right", Qt::Key_Right);
	accel->insertItem(i18n("Toggle Gear"), "Gear", Qt::Key_G);

	gamePause->plugAccel(accel, true);

	accel->insertItem(i18n("Acknowledge that the message has been read", "Acknowledge Message"), "Acknowledge", Qt::Key_Space);

	accel->readSettings();
#else
	gameActions = new KActionCollection (this);

	(void) new KAction (i18n("Accelerate Upwards"), Qt::Key_Up, 0, 0, gameActions,  "Upx");
	(void) new KAction (i18n("Accelerate Leftwards"), KShortcut (Qt::Key_Left), 0, 0, gameActions,  "Left");
	(void) new KAction (i18n("Accelerate Downwards"), KShortcut (Qt::Key_Down), 0, 0, gameActions,  "Down");
	(void) new KAction (i18n("Accelerate Rightwards"), KShortcut (Qt::Key_Right), 0, 0, gameActions,  "Right");
	(void) new KAction (i18n("Toggle Gear"), KShortcut (Qt::Key_G), this, SLOT (slotGear ()), gameActions,  "Gear");
	(void) new KAction (i18n("Acknowledge that the message has been read", "Acknowledge Messages"), KShortcut (Qt::Key_Space), this, SLOT (slotAcknowledge ()), gameActions,  "Acknowledge");

	gameActions->readShortcutSettings ();
	actionCollection ()->readShortcutSettings ();

	installEventFilter (this);
#endif

	readKeys ();

	createGUI("taxipilotui.rc");
}

TaxipilotApp::~TaxipilotApp()
{
#if KDE_IS_VERSION (3, 1, 0)
	delete gameActions;
#endif
}

#if !KDE_IS_VERSION (3, 1, 0)
void TaxipilotApp::keyPressEvent(QKeyEvent * e)
{
	if (!actions.contains (e->key())) {
		e->ignore();
		return;
	}

	Action a = actions[e->key ()];

	switch (a) {
	case Left:
		tp->current_key_lr = -1;
		break;
	case Up:
		tp->current_key_ud = -1;
		break;
	case Right:
		tp->current_key_lr = 1;
		break;
	case Down:
		tp->current_key_ud = 1;
		break;
	case Acknowledge:
		tp->message_handler->acknowledged ();
		break;
	case Gear:
		tp->toggle_gear = true;
		break;
	default:{
			e->ignore();
		}
	}
}

/** We need this in combination with the above handler to work around slow auto-repeats:
	pressed keys are stored in (int) tp->current_key_lr (left is -1, right is 1), tp->current_key_ud (up is -1, down is 1),
	and tp->space_pressed (bool) until they get released. Current key is
	parsed by do_chores. */
void TaxipilotApp::keyReleaseEvent(QKeyEvent * e)
{
	if (!actions.contains (e->key())) {
		e->ignore();
		return;
	}

	Action a = actions[e->key ()];

	switch (a) {
	case Left:
		if (tp->current_key_lr == -1) {
			tp->current_key_lr = 0;
		}
		break;
	case Up:
		if (tp->current_key_ud == -1) {
			tp->current_key_ud = 0;
		}
		break;
	case Right:
		if (tp->current_key_lr == 1) {
			tp->current_key_lr = 0;
		}
		break;
	case Down:
		if (tp->current_key_ud == 1) {
			tp->current_key_ud = 0;
		}
		break;
	case Gear:
		tp->toggle_gear = false;
		break;
	default:{
			e->ignore();
		}
	}
}

#else
bool TaxipilotApp::eventFilter(QObject *, QEvent *event) {
	if (event->type() == QEvent::AccelOverride) {
		KKey key (static_cast<QKeyEvent*> (event));

		if (SUp.contains (key)) {
			tp->current_key_ud = -1;
		} else if (SDown.contains (key)) {
			tp->current_key_ud = 1;
		} else if (SLeft.contains (key)) {
			tp->current_key_lr = -1;
		} else if (SRight.contains (key)) {
			tp->current_key_lr = 1;
		} else {
			return false;
		}
		return true;
	} else if (event->type() == QEvent::KeyRelease) {
		KKey key (static_cast<QKeyEvent*> (event));

		if (SUp.contains (key)) {
			if (tp->current_key_ud == -1) {
				tp->current_key_ud = 0;
			}
		} else if (SDown.contains (key)) {
			if (tp->current_key_ud == 1) {;
				tp->current_key_ud = 0;
			}
		} else if (SLeft.contains (key)) {
			if (tp->current_key_lr == -1) {
				tp->current_key_lr = 0;
			}
		} else if (SRight.contains (key)) {
			if (tp->current_key_lr == 1) {
				tp->current_key_lr = 0;
			}
		} else {
			return false;
		}
		return true;
	}
	return false;
}
#endif

/** Calls up the key-Bindings-Dialog and sanity-checks
the settings */
void TaxipilotApp::configureKeys()
{

	bool ok;

	do {
		ok = true;
#if !KDE_IS_VERSION (3, 1, 0)
		KKeyDialog::configureKeys(accel, true, this);
		if ((!accel->currentKey("Upx")) || (!accel->currentKey("Down")) || (!accel->currentKey("Left"))
			|| (!accel->currentKey("Right")) || (!accel->currentKey("Acknowledge")) || (!accel->currentKey("Gear"))) {
			ok = false;
			KMessageBox::sorry(this, i18n("The acceleration, landing gear, and acknowledge-keys must be defined!"));
		}
#else
		KKeyDialog *dlg = new KKeyDialog (true);
		dlg->insert (actionCollection ());
		dlg->insert (gameActions);
		dlg->configure (true);
//		KKeyDialog::configure (actionCollection ());
		delete dlg;
		gameActions->writeShortcutSettings ();
		actionCollection ()->writeShortcutSettings ();
		if (gameActions->action ("Upx")->shortcut ().isNull () || gameActions->action ("Down")->shortcut ().isNull () ||
			gameActions->action ("Left")->shortcut ().isNull () || gameActions->action ("Right")->shortcut ().isNull () ||
			gameActions->action ("Acknowledge")->shortcut ().isNull () || gameActions->action ("Gear")->shortcut ().isNull ()) {
			ok = false;
			KMessageBox::sorry(this, i18n("The acceleration, landing gear, and acknowledge-keys must be defined!"));
		}
#endif
	} while (!ok);
	readKeys ();
}

/** No descriptions */
void TaxipilotApp::changePrefs(){
	Preferences *prefs = new Preferences ();
	if ( prefs->exec() )
		// update everything
	delete prefs;
}

/** No descriptions */
bool TaxipilotApp::queryExit (){
	delete tp;
	return true;
}

/** Reads the Acclerator-keys into a QMap. Called at initialization and from configureKeys (). */
void TaxipilotApp::readKeys (){
#if !KDE_IS_VERSION (3, 1, 0)
	actions.clear ();

	actions.insert (accel->currentKey("Upx"), Up);
	actions.insert (accel->currentKey("Down"), Down);
	actions.insert (accel->currentKey("Left"), Left);
	actions.insert (accel->currentKey("Right"), Right);
	actions.insert (accel->currentKey("Gear"), Gear);
	actions.insert (accel->currentKey("Acknowledge"), Acknowledge);
	ack_key_name = KAccel::keyToString(accel->currentKey("Acknowledge"));
#else
	SUp = gameActions->action ("Upx")->shortcut ();
	SDown = gameActions->action ("Down")->shortcut ();
	SLeft = gameActions->action ("Left")->shortcut ();
	SRight = gameActions->action ("Right")->shortcut ();
	ack_key_name = gameActions->action ("Acknowledge")->shortcut ().toString ();
#endif

}

/** This slot is in here and not in the Taxi-class, since the Taxi-class is not QObject-derived. */
void TaxipilotApp::slotGear () {
	tp->toggle_gear = true;
}

/** This slot is in here and not in the Message-class, because the message-class is not QObject-derived. */
void TaxipilotApp::slotAcknowledge () {
	tp->message_handler->acknowledged ();
}

void TaxipilotApp::tipOfDay () {
	showTipOfDay (true, this);
}

void TaxipilotApp::showTipOfDay (bool force, QWidget *parent) {
	KTipDialog::showTip (parent, "taxipilot/tips", force);		// show tip of the day
}

#include "taxipilotapp.moc"
